@@ -1,3 +1,6 @@
+"""
+Rates module tests
+"""
 import datetime
 import uuid
 from datetime import date
@@ -15,10 +18,16 @@ from .serializers import RateAmountSerializer
 
 
 class RateTest(TestCase):
+    """
+    Rate object tests
+    """
     base_currency = 'EUR'
     currency = 'USD'
 
     def setUp(self) -> None:
+        """
+        Setup test environment
+        """
         settings.RATE_SERVICE = 'forex'
         self.user, created = User.objects.get_or_create(
             username='test',
@@ -59,19 +68,31 @@ class RateTest(TestCase):
         ]
 
     def test_fetch_rates(self):
+        """
+        Test fetching of rates from service
+        """
         rates = Rate.objects.fetch_rates(base_currency=self.base_currency)
         self.assertIsNotNone(rates)
 
     def test_fetch_rates_with_date(self):
+        """
+        Test fetching of rates at a given date
+        """
         rates = Rate.objects.fetch_rates(base_currency=self.base_currency,
                                          date_obj=date(year=2020, month=6, day=1))
         self.assertIsNotNone(rates)
 
     def test_fetch_rate(self):
+        """
+        Test fectch of a specific conversion rate
+        """
         rate = Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         self.assertIsNotNone(rate)
 
     def test_fetch_rate_with_date(self):
+        """
+        Test of specific conversion rate at a given date
+        """
         rate = Rate.objects.fetch_rates(
             base_currency=self.base_currency, currency=self.currency,
             date_obj=date(year=2020, month=6, day=1)
@@ -79,17 +100,26 @@ class RateTest(TestCase):
         self.assertIsNotNone(rate)
 
     def test_find_rate(self):
+        """
+        Find a rate between currencies
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         rate = Rate.objects.find_rate(base_currency=self.base_currency, currency=self.currency)
         self.assertIsNotNone(rate, msg="no rate found")
 
     def test_find_pivot_rate(self):
+        """
+        Find rate between currencies that are not directly linked
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         Rate.objects.fetch_rates(base_currency=self.currency, currency='AUD')
         rate = Rate.objects.find_rate(base_currency=self.base_currency, currency='AUD')
         self.assertIsNotNone(rate, msg="no rate found")
 
     def test_rate_at_date(self):
+        """
+        Find rate at a specific date
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         Rate.objects.fetch_rates(base_currency=self.currency, currency='AUD')
         rate = Rate.objects.find_rate(base_currency=self.base_currency, currency=self.currency)
@@ -98,6 +128,9 @@ class RateTest(TestCase):
         self.assertIsNotNone(rate.pk, msg="no pivot rate found")
 
     def test_custom_rate(self):
+        """
+        Create a custom rate
+        """
         Rate.objects.create(
             user=self.user,
             key=self.key,
@@ -110,6 +143,9 @@ class RateTest(TestCase):
             Rate.objects.filter(user=self.user, key=self.key, base_currency='AFN', currency='AUD'))
 
     def test_find_rate_chain(self):
+        """
+        Test find rate with keys
+        """
         Rate.objects.create(
             user=self.user,
             key=self.key,
@@ -139,6 +175,9 @@ class RateTest(TestCase):
         self.assertEqual(rates, ['AUD', 'AFN', 'JPY', 'BND'])
 
     def test_find_mixed_rate_chain(self):
+        """
+        Test find rate with keys with gaps in the key chain
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency)
         Rate.objects.create(
             user=self.user,
@@ -169,6 +208,9 @@ class RateTest(TestCase):
         self.assertEqual(rates, ['ARS', 'AFN', 'BND', 'JPY', 'EUR'])
 
     def test_no_rate_path(self):
+        """
+        Test what happens if no rate is found
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency)
         Rate.objects.create(
             user=self.user,
@@ -196,6 +238,9 @@ class RateTest(TestCase):
         )
 
     def test_find_rate_custom(self):
+        """
+        Test conversion between rates with custom rate
+        """
         Rate.objects.create(
             user=self.user,
             key=self.key,
@@ -228,6 +273,9 @@ class RateTest(TestCase):
         self.assertEqual(rate.value, 0.123 * 200 * 13)
 
     def test_find_rate_mixed(self):
+        """
+        Test conversion between rates with custom and standard rates
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency)
         Rate.objects.create(
             user=self.user,
@@ -266,6 +314,9 @@ class RateTest(TestCase):
         self.assertEqual(rate.value, 0.123 * 200 * 13 * jpy_eur.value)
 
     def test_find_rate_override(self):
+        """
+        Test having 2 rates for the same period, with a custom and a standard one
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency)
         Rate.objects.create(
             user=self.user,
@@ -312,7 +363,61 @@ class RateTest(TestCase):
             key=self.key)
         self.assertEqual(rate.value, 0.123 * 200 * 13 * jpy_eur_custom.value)
 
+
+class RateAPITest(TestCase):
+    """
+    Rate object tests
+    """
+    base_currency = 'EUR'
+    currency = 'USD'
+
+    def setUp(self) -> None:
+        """
+        Setup test environment
+        """
+        settings.RATE_SERVICE = 'forex'
+        self.user, created = User.objects.get_or_create(
+            username='test',
+            email='test@ipd.com'
+        )
+        self.user.set_password('test')
+        self.user.save()
+        Token.objects.create(user=self.user)
+        self.key = uuid.uuid4()
+        self.amounts = [
+            {
+                'currency': 'USD',
+                'amount': 100,
+                'date_obj': '2020-07-22'
+            },
+            {
+                'currency': 'AUD',
+                'amount': 50,
+                'date_obj': '2020-07-23'
+            },
+        ]
+        self.trash_amounts = [
+            {
+                'currency': 'USD',
+                'amount': 'toto',
+                'date_obj': '01/01/2020'
+            },
+            {
+                'currency': 'LOL',
+                'date_obj': '2020-07-23'
+            },
+            {
+                'date_obj': '2020-07-23'
+            },
+            {
+                'currency': 'JPY',
+            },
+        ]
+
     def test_post_rate(self):
+        """
+        Test rate creation
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -336,6 +441,9 @@ class RateTest(TestCase):
             self.assertEqual(len(response.json()), 2)
 
     def test_post_rate_without_key(self):
+        """
+        Test standard rate creation without key
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -352,6 +460,9 @@ class RateTest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
 
     def test_list_request(self):
+        """
+        Test rate list
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         client = APIClient()
         response = client.get(
@@ -360,6 +471,9 @@ class RateTest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
 
     def test_stats_request(self):
+        """
+        Test rate stats
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         client = APIClient()
         response = client.get(
@@ -369,6 +483,9 @@ class RateTest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
 
     def test_connected_list_request(self):
+        """
+        Test rates list for authenticated user
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
@@ -400,39 +517,10 @@ class RateTest(TestCase):
         else:
             self.assertEqual(len(response.json()), len(anon_response.json()) + 2)
 
-    def test_list_user_request(self):
-        Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
-        client = APIClient()
-        token = Token.objects.get(user__username=self.user.username)
-        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
-        post_response = client.post(
-            '/rates/',
-            data={
-                'key': self.key,
-                'currency': 'USD',
-                'base_currency': 'EUR',
-                'value_date': '2020-01-01',
-                'value': 1.10,
-            }
-        )
-        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
-        self.assertIn('id', post_response.json())
-        response = client.get(
-            '/rates/',
-            format='json')
-        anon_client = APIClient()
-        anon_response = anon_client.get(
-            '/rates/',
-            format='json')
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertEqual(anon_response.status_code, status.HTTP_200_OK)
-        if 'results' in response.json():
-            self.assertEqual(len(response.json()['results']), len(anon_response.json()['results']) + 2)
-        else:
-            self.assertEqual(len(response.json()), len(anon_response.json()) + 2)
-
-
     def test_list_with_key_request(self):
+        """
+        Test list with key argument
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -457,6 +545,9 @@ class RateTest(TestCase):
             self.assertEqual(response.json()[0]['key'], str(self.key))
 
     def test_list_with_key_or_null_request(self):
+        """
+        Test key_or_null filter
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -482,6 +573,9 @@ class RateTest(TestCase):
             self.assertEqual(len(response.json()), 3)
 
     def test_list_with_key_isnull_request(self):
+        """
+        Test key_isnull filter
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -507,6 +601,9 @@ class RateTest(TestCase):
             self.assertEqual(response.json()[0]['key'], None)
 
     def test_list_with_key_and_currency_request(self):
+        """
+        Test key and currency filters
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -531,6 +628,9 @@ class RateTest(TestCase):
             self.assertEqual(len(response.json()), 1)
 
     def test_stats_with_key_and_currency_request(self):
+        """
+        Test stats with key and currency filters
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -563,6 +663,9 @@ class RateTest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
 
     def test_list_with_key_and_base_currency_request(self):
+        """
+        Test stats with key and base_currency filters
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -587,13 +690,22 @@ class RateTest(TestCase):
             self.assertEqual(len(response.json()), 1)
 
     def test_retrieve_request(self):
+        """
+        Test retreiving a specific rate
+        """
+        Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
+        rate = Rate.objects.first()
         client = APIClient()
         response = client.get(
-            '/rates/',
+            f'/rates/{rate.pk}/',
             format='json')
         self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(response.json()['id'], rate.pk)
 
     def test_bulk_create_request(self):
+        """
+        Test of bulk creation
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -615,6 +727,9 @@ class RateTest(TestCase):
                                                                                    day=1)).days + 1)
 
     def test_latest_currency_request(self):
+        """
+        Test filtering latest values for currency
+        """
         client = APIClient()
         token = Token.objects.get(user__username=self.user.username)
         client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
@@ -647,10 +762,17 @@ class RateTest(TestCase):
 
 
 class RateConverterTest(TestCase):
+    """
+    Test Rate conversion
+    """
+
     base_currency = 'EUR'
     currency = 'USD'
 
     def setUp(self) -> None:
+        """
+        setup test environment
+        """
         settings.RATE_SERVICE = 'forex'
         from_date = datetime.date(year=2020, month=7, day=20)
         to_date = datetime.date(year=2020, month=7, day=25)
@@ -692,9 +814,15 @@ class RateConverterTest(TestCase):
         ]
 
     def test_created(self):
+        """
+        Test converter initial status
+        """
         self.assertEqual(self.converter.status, self.converter.INITIATED_STATUS)
 
     def test_add_data(self):
+        """
+        Test adding data to a converter
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         errors = self.converter.add_data(self.amounts)
         self.assertEqual(errors, [])
@@ -703,6 +831,9 @@ class RateConverterTest(TestCase):
         self.assertIsNotNone(cache.get(self.converter.id))
 
     def test_trash_amounts(self):
+        """
+        Test adding trash data to the converter
+        """
         converter = RateConverter(user=self.user, base_currency='EUR')
         errors = converter.add_data(self.trash_amounts)
         self.assertEqual(len(errors), 4)
@@ -712,6 +843,9 @@ class RateConverterTest(TestCase):
         self.assertNotIn("currency", errors[3])
 
     def test_convert(self):
+        """
+        Conversion test
+        """
         result = self.converter.convert()
         self.assertEqual(result.id, self.converter.id)
         self.assertEqual(result.target, 'EUR')
@@ -722,6 +856,9 @@ class RateConverterTest(TestCase):
         self.assertEqual(result.sum, converted_sum)
 
     def test_convert_pivot(self):
+        """
+        Test converting currencies with indirect relation
+        """
         Rate.objects.fetch_rates(base_currency='EUR', currency='AUD',
                                  date_obj=datetime.date(year=2020, month=7, day=23))
         Rate.objects.fetch_rates(base_currency='JPY', currency='EUR',
@@ -744,7 +881,63 @@ class RateConverterTest(TestCase):
         converted_sum = sum([d.converted_value for d in result.detail])
         self.assertEqual(result.sum, converted_sum)
 
+
+class RateConverterAPITest(TestCase):
+    """
+    Test Rate conversion API
+    """
+
+    base_currency = 'EUR'
+    currency = 'USD'
+
+    def setUp(self) -> None:
+        """
+        setup test environment
+        """
+        settings.RATE_SERVICE = 'forex'
+        from_date = datetime.date(year=2020, month=7, day=20)
+        to_date = datetime.date(year=2020, month=7, day=25)
+        Rate.objects.fetch_rates(base_currency=self.base_currency, date_obj=from_date,
+                                 to_obj=to_date)
+        self.user, created = User.objects.get_or_create(
+            username='test',
+            email='test@ipd.com'
+        )
+        self.converter = RateConverter(user=self.user, base_currency='EUR')
+        self.amounts = [
+            {
+                'currency': 'USD',
+                'amount': 100,
+                'date_obj': '2020-07-22'
+            },
+            {
+                'currency': 'AUD',
+                'amount': 50,
+                'date_obj': '2020-07-23'
+            },
+        ]
+        self.trash_amounts = [
+            {
+                'currency': 'USD',
+                'amount': 'toto',
+                'date_obj': '01/01/2020'
+            },
+            {
+                'currency': 'LOL',
+                'date_obj': '2020-07-23'
+            },
+            {
+                'date_obj': '2020-07-23'
+            },
+            {
+                'currency': 'JPY',
+            },
+        ]
+
     def test_convert_request(self):
+        """
+        Test conversion request
+        """
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         Rate.objects.fetch_rates(base_currency=self.currency, currency='AUD')
         amounts = RateAmountSerializer(self.amounts, many=True)
@@ -761,6 +954,9 @@ class RateConverterTest(TestCase):
         self.assertEqual(len(response.json().get('detail')), len(self.amounts))
 
     def test_convert_batch_request(self):
+        """
+        Test batch conversion
+        """
         batch_id = uuid.uuid4()
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         Rate.objects.fetch_rates(base_currency=self.currency, currency='AUD')
@@ -791,6 +987,9 @@ class RateConverterTest(TestCase):
         self.assertEqual(len(response.json().get('detail')), 2 * len(self.amounts))
 
     def test_watch_request(self):
+        """
+        Test observation of the batch
+        """
         batch_id = uuid.uuid4()
         Rate.objects.fetch_rates(base_currency=self.base_currency, currency=self.currency)
         Rate.objects.fetch_rates(base_currency=self.currency, currency='AUD')
